#![no_std]
#![no_main]

extern crate alloc;

pub mod constants;
pub mod entry_points;
mod error;

use alloc::{
    collections::BTreeMap,
    format,
    string::{String, ToString},
    vec,
    vec::Vec,
};

//use casper_contract::contract_api::{self};
use casper_contract::{
    contract_api::{
        runtime::{self, get_caller, get_key, get_named_arg, put_key, revert},
        storage::{self, dictionary_put},
    },
    unwrap_or_revert::UnwrapOrRevert,
};
use casper_types::{
    api_error::ApiError, bytesrepr::ToBytes, contracts::NamedKeys, runtime_args, CLValue,
    ContractHash, ContractPackageHash, Key, RuntimeArgs, URef, U256,
};
use entry_points::generate_entry_points;

const CONTRACT_PACKAGE_NAME: &str = "package_name";
const CONTRACT_ACCESS_UREF: &str = "access_uref";

const CONTRACT_VERSION_KEY: &str = "version";
const CONTRACT_KEY: &str = "availability";
const MESSAGE_KEY: &str = "message";

#[no_mangle]
pub extern "C" fn hello() {
    let message: String = runtime::get_named_arg("message");
    let uref: URef = runtime::get_key(MESSAGE_KEY)
        .unwrap_or_revert_with(ApiError::MissingKey)
        .into_uref()
        .unwrap_or_revert_with(ApiError::UnexpectedKeyVariant);
    storage::write(uref, message);
}

#[no_mangle]
pub extern "C" fn query() {
    let uref: URef = runtime::get_key(MESSAGE_KEY)
        .unwrap_or_revert_with(ApiError::MissingKey)
        .into_uref()
        .unwrap_or_revert_with(ApiError::UnexpectedKeyVariant);
    let result: String = storage::read(uref)
        .unwrap_or_revert_with(ApiError::Read)
        .unwrap_or_revert_with(ApiError::ValueNotFound);
    let typed_result = CLValue::from_t(result).unwrap_or_revert();
    runtime::ret(typed_result); // Return the message value.
}

#[no_mangle]
pub extern "C" fn stake_availability() {}

#[no_mangle]
pub extern "C" fn assign_work() {}

#[no_mangle]
pub extern "C" fn get_assigned_work() {}

// Entry point that executes automatically when a caller installs the contract.
#[no_mangle]
pub extern "C" fn call() {
    let message = storage::new_uref("initial message");
    // In the named keys of the contract, add a key for the message
    let mut named_keys = NamedKeys::new();
    let key_name = String::from(MESSAGE_KEY);
    named_keys.insert(key_name, message.into());

    // Create the entry points for this contract.
    let entry_points = generate_entry_points();

    // Create a new contract package that can be upgraded.
    let (stored_contract_hash, contract_version) = storage::new_contract(
        entry_points,
        Some(named_keys),
        Some(CONTRACT_PACKAGE_NAME.to_string()),
        Some(CONTRACT_ACCESS_UREF.to_string()),
    );

    /* To create a locked contract instead, use new_locked_contract and throw away the contract version returned.
    let (stored_contract_hash, _) =
        storage::new_locked_contract(counter_entry_points, Some(counter_named_keys), None, None); */

    // Store the contract version in the context's named keys.
    let version_uref = storage::new_uref(contract_version);
    runtime::put_key(CONTRACT_VERSION_KEY, version_uref.into());

    // Create a named key for the contract hash.
    runtime::put_key(CONTRACT_KEY, stored_contract_hash.into());
}
